;* Title	: USB ADC ADCU14-8D
;* Version	: 2.00
;* Target	: ATmega8
;* Author	: wubblick@yahoo.com

.include "m8def.inc"
.include "macros.mac"
.include "Header.asm"

;----------------------------------------------------------------------------

.CSEG	;Code segment

;----------------------------------------------------------------------------

;Interrupt vectors:

.org	0
	rjmp	Init			;reset vector
.org	OVF0addr
	rjmp	Timer			;system timer vector
.org	SPMRaddr+1			;skip vectors table

;----------------------------------------------------------------------------

;Program execution starts here:

Init:	ldy	RAMEND	
	out	SPL,YL			;locate stack
	out	SPH,YH
	rcall	iWdog			;start internal watchdog
	rcall	iPorts			;ports init
	rcall	iVar			;variables init
	rcall	iTimer			;system timer init
	rcall	iDev			;device control module init
	rcall	iWake			;WAKE environment init
	sei				;interrupts enable

;Main loop:

MAIN:	rcall	mCmd			;execute PC commands
	rcall	mDev			;device module main task
	rcall	mWdog			;watchdogs restart
	rjmp 	MAIN			;loop

;----------------------------------------------------------------------------

;Internal watchdog init:

iWdog:	wdr
	ldi	temp,(1<<WDCE) | (1<<WDE)
	out	WDTCR,temp	
	ldi	temp,(1<<WDE) | (1<<WDP2) | (1<<WDP0)	
	out	WDTCR,temp
	ret	

;----------------------------------------------------------------------------

;Watchdogs restart:

mWdog:	bbrc	Flags,fUpd,mw0
	clbr	Flags,fUpd
	wdr				;internal watchdog restart
mw0:	ret

;----------------------------------------------------------------------------

;Ports init:
	
iPorts:	ldi	temp,PUPB
	out	PORTB,temp		;Init PORTB and on/off pullup
	ldi	temp,DIRB	
	out	DDRB,temp		;Set PORTB direction

	ldi	temp,PUPC
	out	PORTC,temp		;Init PORTC and on/off pullup
	ldi	temp,DIRC
	out	DDRC,temp		;Set PORTC direction

	ldi	temp,PUPD
	out	PORTD,temp		;Init PORTD and on/off pullup
	ldi	temp,DIRD
	out	DDRD,temp		;Set PORTD direction

	ret

;----------------------------------------------------------------------------

;Variables init:

iVar:	clr	Flags			;clear flags
	ret

;----------------------------------------------------------------------------

;System timer init (timer 0):

iTimer:	ldi	temp,(1<<CS02) | (1<<CS00)	
	out	TCCR0,temp		;CK/1024 for TC0

	in	temp,TCNT0		;read timer
	subi	temp,T0Div		;TC0 + (256 - T0Div)
	out	TCNT0,temp		;timer load

	in	temp,TIMSK
	ori	temp,(1<<TOIE0)
	out	TIFR,temp		;clear pending interrupts
	out	TIMSK,temp		;enable TC0 overflow interrupt
	ret

;----------------------------------------------------------------------------

;System timer interrupt:

Timer:	push	temp
	in	temp,SREG		;save status register
	push	temp
	push	YL
	push	YH

	in	temp,TCNT0		;read timer
	subi	temp,T0Div		;TC0 + (256 - T0Div)
	out	TCNT0,temp		;timer reload

	stbr	Flags,fUpd		;set update flag

	rcall	tDev			;device module timed task

	pop	YH
	pop	YL
	pop	temp
	out	SREG,temp		;restore status register
	pop	temp
	reti

;----------------------------------------------------------------------------

.include "Wake.asm"			;link WAKE support module
.include "Cmd.asm"			;link PC commands processing module
.include "Dev.asm"			;link device control module

;----------------------------------------------------------------------------
