;ADCU14-8D PC commands process module

;----------------------------------------------------------------------------

;Command codes:

.equ	C_Nop	= 0x00			;no operation
.equ	C_Err	= 0x01			;get error code
.equ	C_Echo	= 0x02			;get echo
.equ	C_Info	= 0x03			;get device information

.equ	C_GetA	= 0x04			;get ADC
.equ	C_SetA	= 0x05			;set DAC
.equ	C_GetD	= 0x06			;get DIO
.equ	C_SetD	= 0x07			;set DIO
.equ	C_SetW	= 0x08			;set window

;----------------------------------------------------------------------------

.CSEG	;Code segment

;----------------------------------------------------------------------------

;Process commands:

mCmd:	cpi	CMD,C_Nop
	breq	Cmdr

	rcall	ALed			;pulse ACTIVE LED
	cpi	CMD,C_Err
	brne	Cmd1
	rcall	D_Err
	rjmp	Cmdr
Cmd1:	cpi	CMD,C_Echo
	brne	Cmd2
	rcall	D_Echo
	rjmp	Cmdr
Cmd2:	cpi	CMD,C_Info
	brne	Cmd3
	rcall	D_Info
	rjmp	Cmdr

Cmd3:	cpi	CMD,C_GetA
	brne	Cmd4
	rcall	D_GetA
	rjmp	Cmdr
Cmd4:	cpi	CMD,C_SetA
	brne	Cmd5
	rcall	D_SetA
	rjmp	Cmdr
Cmd5:	cpi	CMD,C_GetD
	brne	Cmd6
	rcall	D_GetD
	rjmp	Cmdr
Cmd6:	cpi	CMD,C_SetD
	brne	Cmd7
	rcall	D_SetD
	rjmp	Cmdr
Cmd7:	cpi	CMD,C_SetW
	brne	Cmd8
	rcall	D_SetW
	rjmp	Cmdr
Cmd8:
Cmdr:	ret

;----------------------------------------------------------------------------

;Command "C_Err" processing:

;Tx_Dat[0]	- error code (Err_Tx)

D_Err:	ldi	temp,Err_Tx		;load Err_Tx error code
	ldi	Cnt,1			;N
	sts	Tx_Str+x_Nbt,Cnt  	
	sts	Tx_Str+x_Cmd,CMD  	;command
	sts	Tx_Str+x_Dat,temp 	;error code
	rcall	UARTTX		  	;begin frame TX
	ldi	CMD,C_Nop	  	;end of command
	ret

;----------------------------------------------------------------------------

;Command "C_Echo" processing:

;Tx_Dat[]	- echo bytes

D_Echo:	ldy	Rx_Str
	ldd	temp,Y+x_Nbt		;N
	push	temp
	tst	temp
	breq	ech_no

	mov	Cnt,temp
	ldy	Rx_Dat
	ldz	Tx_Dat
ech_c:	ld	temp,Y+
	st	Z+,temp
	dec	Cnt
	brne	ech_c

ech_no:	pop	Cnt			;N	
	sts	Tx_Str+x_Nbt,Cnt
	sts	Tx_Str+x_Cmd,CMD  	;command
	rcall	UARTTX		  	;begin frame TX
	ldi	CMD,C_Nop	  	;end of command
	ret

;----------------------------------------------------------------------------

;Command "C_Info" processing:

;Tx_Dat[]	- info string

D_Info:	table	Info
	ldy	Tx_Dat
	clr	Cnt
inf_c:	lpm	temp,Z+
	st	Y+,temp
	inc	Cnt
	tst	temp
	brne	inf_c			;Cnt = N

	sts	Tx_Str+x_Nbt,Cnt
	sts	Tx_Str+x_Cmd,CMD  	;command
	rcall	UARTTX		  	;begin frame TX
	ldi	CMD,C_Nop	  	;end of command
	ret
	
;----------------------------------------------------------------------------

;Command "C_GetA" processing:

;Rx_Dat[0]	- channel number (0..7)
;Rx_Dat[1]	- gain code (1,2,5,10,20,50,100)
;Tx_Dat[0]	- error code (Err_No, Err_Pa)
;Tx_Dat[1,2]	- ADC code

D_GetA:	lds	temp,Rx_Dat+0		;load channel number
	rcall	SetCh			;set channel
	brcs	gaerr			;branch if error
	
	lds	temp,Rx_Dat+1		;load gain code
	rcall	SetGn			;set gain
	brcs	gaerr			;branch if error

	rcall	GetADC			;get ADC
	sts	Tx_Dat+1,tempL		;save ADC code low
	sts	Tx_Dat+2,tempM		;save ADC code high
	ldi	temp,Err_No		;no error
	ldi	Cnt,3			;N
	rjmp	gatx

gaerr:	ldi	temp,Err_Pa		;params error
	ldi	Cnt,1			;N

gatx:	sts	Tx_Str+x_Nbt,Cnt
	sts	Tx_Str+x_Cmd,CMD  	;command
	sts	Tx_Str+x_Dat,temp 	;error code
	rcall	UARTTX		  	;begin frame TX
	ldi	CMD,C_Nop	  	;end of command
	ret

;----------------------------------------------------------------------------

;Command "C_SetA" processing:

;Rx_Dat[0]	- channel number (0..3)
;Rx_Dat[1,2]	- DAC code
;Tx_Dat[0]	- error code (Err_No, Err_Pa)

D_SetA:	lds	temp,Rx_Dat+0		;load channel number
	lds	tempL,Rx_Dat+1		;load DAC code low
	lds	tempH,Rx_Dat+2		;load DAC code high
	rcall	SetDAC			;set DAC
	brcs	dacerr			;branch if error
	ldi	temp,Err_No		;no error
	rjmp	dactx
dacerr:	ldi	temp,Err_Pa		;params error
dactx:	ldi	Cnt,1			;N
	sts	Tx_Str+x_Nbt,Cnt
	sts	Tx_Str+x_Cmd,CMD  	;command
	sts	Tx_Str+x_Dat,temp 	;error code
	rcall	UARTTX		  	;begin frame TX
	ldi	CMD,C_Nop	  	;end of command
	ret

;----------------------------------------------------------------------------

;Command "C_GetD" processing:

;Tx_Dat[0]	- error code (Err_No)
;Tx_Dat[1..3]	- DIO code

D_GetD:	rcall	GetDIO			;get DIO
	sts	Tx_Dat+1,tempL		;save DIO code low
	sts	Tx_Dat+2,tempM		;save DIO code middle
	sts	Tx_Dat+3,tempH		;save DIO code high
	ldi	temp,Err_No
	ldi	Cnt,4			;N
	sts	Tx_Str+x_Nbt,Cnt
	sts	Tx_Str+x_Cmd,CMD  	;command
	sts	Tx_Str+x_Dat,temp 	;error code
	rcall	UARTTX		  	;begin frame TX
	ldi	CMD,C_Nop	  	;end of command
	ret

;----------------------------------------------------------------------------

;Command "C_SetD" processing:

;Rx_Dat[0..2]	- DIO code
;Tx_Dat[0]	- error code (Err_No)

D_SetD:	lds	tempL,Rx_Dat+0		;load DIO code low	
	lds	tempM,Rx_Dat+1		;load DIO code middle
	lds	tempH,Rx_Dat+2		;load DIO code high
	rcall	SetDIO			;set DIO
	ldi	temp,Err_No
	ldi	Cnt,1			;N
	sts	Tx_Str+x_Nbt,Cnt
	sts	Tx_Str+x_Cmd,CMD  	;command
	sts	Tx_Str+x_Dat,temp 	;error code
	rcall	UARTTX		  	;begin frame TX
	ldi	CMD,C_Nop	  	;end of command
	ret

;----------------------------------------------------------------------------

;Command "C_SetW" processing:

;Rx_Dat[0]	- window length
;Tx_Dat[0]	- error code (Err_No)

D_SetW:	lds	temp,Rx_Dat+0		;load window length
	rcall	SetWin			;set window length
	ldi	temp,Err_No		;no error
	ldi	Cnt,1			;N
	sts	Tx_Str+x_Nbt,Cnt
	sts	Tx_Str+x_Cmd,CMD  	;command
	sts	Tx_Str+x_Dat,temp 	;error code
	rcall	UARTTX		  	;begin frame TX
	ldi	CMD,C_Nop	  	;end of command
	ret

;----------------------------------------------------------------------------

Info:	.DB "ADCU14-8D V2.0", 0, 0

;----------------------------------------------------------------------------
