;ADCU14-8D device module

;----------------------------------------------------------------------------

;Constantes:

.equ	MuxT	= 500		;mux and gain settlind delay [uS]
.equ	AdcT	= 30		;ADC ready delay [uS]
.equ	LedT	= 5		;LED ACTIVE timer [mS]

.equ	ADC_SCALE = 1		;ADC scale: 0 - unipolar, 1 - bipolar

.equ MUXDEL = (((Fclk/1000000)*MuxT)-6)/4
.if  MUXDEL > MAXWORD
.error "out of range constant"
.endif
.equ ADCDEL = (((Fclk/1000000)*AdcT)-6)/4
.if  ADCDEL > MAXWORD
.error "out of range constant"
.endif
.equ	LedTmV = (1000*LedT)/Tsys
.if  LedTmV > MAXBYTE
.error "out of range constant"
.endif

;----------------------------------------------------------------------------

.DSEG	;data segment (internal SRAM)

LedTm:	.byte 1			;LED ACTIVE timer
Win:	.byte 1			;window length

;----------------------------------------------------------------------------

.CSEG	;Code segment

;----------------------------------------------------------------------------

;Init device module:

iDev:	Port_DR_1		;DAC RESET = 1
	ser	tempL
	ser	tempM
	ser	tempH
	rcall	SetDIO		;set DIO to input
	Port_OE_0		;DIO output enable
	ldi	temp,0
	rcall	SetWin		;set window length
	ret

;----------------------------------------------------------------------------

;Device module main task:

mDev:	lds	temp,LedTm	;check LED timer
	tst	temp
	brne	mdret
	Port_LA_0		;LED off
mdret:	ret

;----------------------------------------------------------------------------

;Device timed task:

tDev:	lds	temp,LedTm
	tst	temp
	breq	tdret
	dec	temp
	sts	LedTm,temp
tdret:	ret

;----------------------------------------------------------------------------

;Pulse ACTIVE LED:

ALed:	Port_LA_1		;LED on
	ldi	temp,LedTmV	;start LED timer
	sts	LedTm,temp	
	ret

;----------------------------------------------------------------------------

;Set channel number
;Input:	temp	- channel number (0..7)
;Out:	C = 1 if channel number error

;Input | Ch | A0 | A1 | A2
;-------------------------
;  0   | 7  | 1  | 1  | 1
;  1   | 3  | 1  | 1  | 0
;  2   | 6  | 0  | 1  | 1
;  3   | 2  | 0  | 1  | 0
;  4   | 5  | 1  | 0  | 1
;  5   | 1  | 1  | 0  | 0
;  6   | 4  | 0  | 0  | 1
;  7   | 0  | 0  | 0  | 0


SetCh:	cpi	temp,0
	brne	in1
	Port_A0_1
	Port_A1_1
	Port_A2_1
	rjmp	inok

in1:	cpi	temp,1
	brne	in2
	Port_A0_1
	Port_A1_1
	Port_A2_0
	rjmp	inok

in2:	cpi	temp,2
	brne	in3
	Port_A0_0
	Port_A1_1
	Port_A2_1
	rjmp	inok

in3:	cpi	temp,3
	brne	in4
	Port_A0_0
	Port_A1_1
	Port_A2_0
	rjmp	inok

in4:	cpi	temp,4
	brne	in5
	Port_A0_1
	Port_A1_0
	Port_A2_1
	rjmp	inok

in5:	cpi	temp,5
	brne	in6
	Port_A0_1
	Port_A1_0
	Port_A2_0
	rjmp	inok

in6:	cpi	temp,6
	brne	in7
	Port_A0_0
	Port_A1_0
	Port_A2_1
	rjmp	inok

in7:	cpi	temp,7
	brne	inerr
	Port_A0_0
	Port_A1_0
	Port_A2_0

inok:	clc			;clear error flag	
	rjmp	inret
inerr:	sec			;set error flag
inret:	ret

;----------------------------------------------------------------------------

;Set gain:
;Input:	temp	- Gain code (1,2,5,10,20,50,100)
;Out:	C = 1 if gain code error

;Gain | Stage1 | Stage2 | G0 | G1 | G2 | G3 
;------------------------------------------
; 1   |   1    |   1    | 0  | 0  | 0  | 0
; 2   |   2    |   1    | 1  | 0  | 0  | 0
; 5   |   5    |   1    | 0  | 1  | 0  | 0
; 10  |   5    |   2    | 0  | 1  | 1  | 0
; 20  |   10   |   2    | 1  | 1  | 1  | 0
; 50  |   10   |   5    | 1  | 1  | 0  | 1
; 100 |   10   |   10   | 1  | 1  | 1  | 1

SetGn:	cpi	temp,1
	brne	gn2
	Port_G0_0
	Port_G1_0
	Port_G2_0
	Port_G3_0
	rjmp	gnok	

gn2:	cpi	temp,2
	brne	gn5
	Port_G0_1
	Port_G1_0
	Port_G2_0
	Port_G3_0
	rjmp	gnok	

gn5:	cpi	temp,5
	brne	gn10
	Port_G0_0
	Port_G1_1
	Port_G2_0
	Port_G3_0
	rjmp	gnok	

gn10:	cpi	temp,10
	brne	gn20
	Port_G0_0
	Port_G1_1
	Port_G2_1
	Port_G3_0
	rjmp	gnok	

gn20:	cpi	temp,20
	brne	gn50
	Port_G0_1
	Port_G1_1
	Port_G2_1
	Port_G3_0
	rjmp	gnok	

gn50:	cpi	temp,50
	brne	gn100
	Port_G0_1
	Port_G1_1
	Port_G2_0
	Port_G3_1
	rjmp	gnok	

gn100:	cpi	temp,100
	brne	gnerr
	Port_G0_1
	Port_G1_1
	Port_G2_1
	Port_G3_1

gnok:	clc			;clear error flag
	rjmp	gnret
gnerr:	sec			;set error flag
gnret:	ret

;----------------------------------------------------------------------------

;Start and read ADC:
;Out:	temp:ZH:ZL - ADC code

RdADC:	push	Cnt
	Port_AS_0		;start ADC
	Port_AS_1
	ldz	ADCDEL
	rcall	Delay		;delay for ADC ready

	rcall	Rd7894		;read high byte
	mov	ZH,temp
	rcall	Rd7894		;read low byte
	mov	ZL,temp
	clr	temp

.if ADC_SCALE > 0
	bbrc	ZH,5,plus	;check sign
	ori	ZH,0xC0		;extend sign
	ser	temp
plus:
.endif
	pop	Cnt
	ret

;----------------------------------------------------------------------------

;Get ADC:
;Out:	tempM:tempL - ADC code

GetADC:	ldz	MUXDEL
	rcall	Delay		;delay for settling
	rcall	LdWin		;YH:YL - samples count

	clr	tempL		;clear sum
	clr	tempM
	clr	tempH

gadc:	mov	temp,YL
	or	temp,YH
	breq	adone
	rcall	RdADC
	add	tempL,ZL
	adc	tempM,ZH
	adc	tempH,temp

	subi	YL,1
	sbci	YH,0
	rjmp	gadc

adone:	rcall	LdWin		;YH:YL - samples count
	tst	temp		;check window length
	breq	ares		;zero length, return

	lsr	YH
	ror	YL
	lsr	YH
	ror	YL		;YH:YL - samples count / 4

	bst	tempH,7		;check sign
	bld	Flags,fSign	;store sign
	brtc	div24		;if plus

	com	tempL		;change sign of dividend
	com	tempM		
	com	tempH		
	subi	tempL,byte1(-1)
	sbci	tempM,byte2(-1)
	sbci	tempH,byte3(-1)

;Divide tempH:tempM:tempL / YH:YL = ~tempM:tempL

div24:	clr	temp
	clr	tempA		;clear remainder Low byte
	clr	tempB
	sub	tempC,tempC	;clear remainder High byte and carry
	
	ldi	Cnt,25		;init loop counter
	
div1:	rol	tempL		;shift left dividend
	rol	tempM
	rol	tempH
	dec	Cnt		;decrement counter
	breq	divret
	rol	tempA		;shift dividend into remainder
	rol	tempB
	rol	tempC
	sub	tempA,YL	;remainder = remainder - divisor
	sbc	tempB,YH	
	sbc	tempC,temp	
	brcc	div1		;if result negative
	add	tempA,YL	; restore remainder
	adc	tempB,YH
	adc	tempC,temp
	rjmp	div1

divret:	bbrs	Flags,fSign,minus
	com	tempL
	com	tempM
	rjmp	ares

minus:	subi	tempL,byte1(-1)
	sbci	tempM,byte2(-1)

ares:	ret

;----------------------------------------------------------------------------

;Set DAC:
;Input:	temp - channel number (0..3)
;	tempH:tempL - DAC code
;Out:	C = 1 if channel number error

SetDAC:	cpi	temp,3
	brlo	dnok
	breq	dnok
	rjmp	dnerr
dnok:		
	ldi	Cnt,4
shd:	lsr	tempH
	ror	tempL
	dec	Cnt
	brne	shd

	ldi	Cnt,0x08
	eor	tempH,Cnt	;convert to stright binary
	bst	temp,0
	bld	tempH,6
	bst	temp,1
	bld	tempH,7
	mov	temp,tempH
	rcall	Spi_Wr		;write address and data high
	mov	temp,tempL
	rcall	Spi_Wr		;write data low
	Port_DL_0
	Port_DL_1

	clc			;clear error flag
	rjmp	dnret
dnerr:	sec			;set error flag
dnret:	ret

;----------------------------------------------------------------------------

;Get DIO:
;Out:	tempH:tempM:tempL - DIO code

GetDIO:	Port_SC_0		;SRCLK, RCLK = 0
	Port_SD_0		;SR LOAD = 0
	Port_SC_1		;SRCLK, RCLK = 1	
	Port_SD_1		;SR LOAD = 1
	rcall	Rd597
	mov	tempL,temp
	rcall	Rd597
	mov	tempM,temp
	rcall	Rd597
	mov	tempH,temp
	ret

;----------------------------------------------------------------------------

;Set DIO:
;Input:	tempH:tempM:tempL - DIO code

SetDIO:	Port_OL_0		;RCK = 0
	mov	temp,tempL
	rcall	Wr595
	mov	temp,tempM
	rcall	Wr595
	mov	temp,tempH
	rcall	Wr595
	Port_SD_1		;SER IN = 1
	Port_OL_1		;RCK = 1
	ret

;----------------------------------------------------------------------------

;Set window length:
;Input:	temp - window length, ms

SetWin:	sts	Win,temp
	ret

;----------------------------------------------------------------------------

;Load samples counter:
;Input: Win - window length, ms
;Out:	YH:YL - samples count
;	temp - window length, ms

LdWin:	lds	temp,Win
	tst	temp
	brne	ldw1

	ldi	YL,4		;4 samples for 0 ms window
	ldi	YH,0
	rjmp	ldwr

ldw1:	ldi	YL,20		;samples per ms
	mul	temp,YL
wins:	mov	YL,r0
	mov	YH,r1

ldwr:	ret		

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------

;Read ADC AD7894:
;Out:	temp - data byte

Rd7894:	ldi	Cnt,8		;counter load
ra_bit:	Port_AC_1		;SCLK = 1
	clc			;C = 0
	Skip_if_AD_0
	sec			;C = 1
	rol	temp
	Port_AC_0		;SCLK = 0
	dec	Cnt
	brne	ra_bit
	ret

;----------------------------------------------------------------------------

;Read HC597 register
;Out:	temp - data byte

Rd597:	ldi	Cnt,8		;counter load
rd_bit:	Port_SC_0		;SRCLK = 0
	clc			;C = 0
	Skip_if_ID_0
	sec			;C = 1
	ror	temp
	Port_SC_1		;SRCLK = 1
	dec	Cnt
	brne	rd_bit
	ret

;----------------------------------------------------------------------------

;Write HC595 register
;Input:	temp - data
;Input bit order:  D7, D6, D5, D4, D3, D2, D1, D0
;Output bit order: D3, D2, D1, D0, D4, D5, D6, D7

Wr595:	com	temp		;TPIC6C595 has inversion
	swap	temp		;D3, D2, D1, D0, D7, D6, D5, D4
	mov	Cnt,temp
	bst	Cnt,0
	bld	temp,3
	bst	Cnt,1
	bld	temp,2
	bst	Cnt,2
	bld	temp,1
	bst	Cnt,3
	bld	temp,0
	rcall	Spi_Wr
	ret

;----------------------------------------------------------------------------

;Write byte via SPI:
;Input:	temp - data byte

Spi_Wr:	ldi	Cnt,8		;counter load
wd_bit:	Port_SC_0		;SRCK = 0
	rol	temp
	brcs	wd_1
wd_0:	Port_SD_0		;SER IN = 0 or..
	rjmp	wd_clk
wd_1:	Port_SD_1		;SER IN = 1
wd_clk:	dec	Cnt
	Port_SC_1		;SRCK = 1
	brne	wd_bit
	ret

;----------------------------------------------------------------------------

;Delay (1/Fclk * (6 + 4 * ZH:ZL)):

;	rcall			;3
Delay:	subi	ZL,1		;1
	sbci	ZH,0		;1
	brne	Delay		;2/1
	ret			;4

;----------------------------------------------------------------------------
