;ADCU14-8D header

;***** Constantes:

.equ	Fclk	= 16000000		;Fclk, Hz
.equ	Tsys	= 1000			;time base, uS
.equ	MAXWORD	= 0xFFFF
.equ	MAXBYTE	= 0xFF

.equ T0Div = ((((Fclk/1024)*Tsys)/100000)+5)/10	;CK/1024
.if  T0Div > MAXBYTE
.error "out of range constant"
.endif

;----------------------------------------------------------------------------

;Port Definitions:

.equ	DIRB	= 0b00111111		;Port B direction
.equ	PUPB	= 0b00100000		;Port B pull-ups
.equ	G0	= PB0			; gain select 0
.equ	G1	= PB1			; gain select 1
.equ	G2	= PB2			; gain select 2
.equ	G3	= PB3			; gain select 3    
.equ	LA	= PB4			; LED ACTIVE
.equ	DL	= PB5			; DAC /LREG

.equ	DIRC	= 0b00111101		;Port C direction
.equ	PUPC	= 0b00110011		;Port C pull-ups
.equ	AS	= PC0			; ADC /CONVST
.equ	AD	= PC1			; ADC SDATA
.equ	AC	= PC2			; ADC SCLK
.equ	DR	= PC3			; DAC /RESET
.equ	OL	= PC4			; 595 RCK
.equ	OE	= PC5			; 595 OE

.equ	DIRD	= 0b11101110		;Port D direction
.equ	PUPD	= 0b00011111		;Port D pull-ups
;.equ	RXD	= PD0			; RS-232 RXD
;.equ	TXD	= PD1			; RS-232 TXD
.equ	SD	= PD2			; 595 SER IN, 597 SRLOAD
.equ	SC	= PD3			; 595 SRCK, 597 SRCLK, 597 RCLK
.equ	ID	= PD4			; 597 QH
.equ	A0	= PD5			; input mux address 0
.equ	A1	= PD6			; input mux address 1
.equ	A2	= PD7			; input mux address 2

;***** Port lines control macros:

.macro	Port_G0_0			;G0 = 0
	cbi	PORTB,G0
.endm

.macro	Port_G0_1			;G0 = 1
	sbi	PORTB,G0
.endm

.macro	Port_G1_0			;G1 = 0
	cbi	PORTB,G1
.endm

.macro	Port_G1_1			;G1 = 1
	sbi	PORTB,G1
.endm

.macro	Port_G2_0			;G2 = 0
	cbi	PORTB,G2
.endm

.macro	Port_G2_1			;G2 = 1
	sbi	PORTB,G2
.endm

.macro	Port_G3_0			;G3 = 0
	cbi	PORTB,G3
.endm

.macro	Port_G3_1			;G3 = 1
	sbi	PORTB,G3
.endm

.macro	Port_LA_0			;LA = 0
	cbi	PORTB,LA
.endm

.macro	Port_LA_1			;LA = 1
	sbi	PORTB,LA
.endm

.macro	Port_DL_0			;DL = 0
	cbi	PORTB,DL
.endm

.macro	Port_DL_1			;DL = 1
	sbi	PORTB,DL
.endm

.macro	Port_AS_0			;AS = 0
	cbi	PORTC,AS
.endm

.macro	Port_AS_1			;AS = 1
	sbi	PORTC,AS
.endm

.macro	Skip_if_AD_0			;skip if AD = 0
	sbic	PINC,AD
.endm

.macro	Port_AC_0			;AC = 0
	cbi	PORTC,AC
.endm

.macro	Port_AC_1			;AC = 1
	sbi	PORTC,AC
.endm

.macro	Port_DR_0			;DR = 0
	cbi	PORTC,DR
.endm

.macro	Port_DR_1			;DR = 1
	sbi	PORTC,DR
.endm

.macro	Port_OL_0			;OL = 0
	cbi	PORTC,OL
.endm

.macro	Port_OL_1			;OL = 1
	sbi	PORTC,OL
.endm

.macro	Port_OE_0			;OE = 0
	cbi	PORTC,OE
.endm

.macro	Port_OE_1			;OE = 1
	sbi	PORTC,OE
.endm

.macro	Port_SD_0			;SD = 0
	cbi	PORTD,SD
.endm

.macro	Port_SD_1			;SD = 1
	sbi	PORTD,SD
.endm

.macro	Port_SC_0			;SC = 0
	cbi	PORTD,SC
.endm

.macro	Port_SC_1			;SC = 1
	sbi	PORTD,SC
.endm

.macro	Skip_if_ID_0			;skip if ID = 0
	sbic	PIND,ID
.endm

.macro	Port_A0_0			;A0 = 0
	cbi	PORTD,A0
.endm

.macro	Port_A0_1			;A0 = 1
	sbi	PORTD,A0
.endm

.macro	Port_A1_0			;A1 = 0
	cbi	PORTD,A1
.endm

.macro	Port_A1_1			;A1 = 1
	sbi	PORTD,A1
.endm

.macro	Port_A2_0			;A2 = 0
	cbi	PORTD,A2
.endm

.macro	Port_A2_1			;A2 = 1
	sbi	PORTD,A2
.endm

;----------------------------------------------------------------------------

;Global Register Variables:

;r0, r1 used with mul instruction

.def	tempA	= r2			;temporary register tempA
.def	tempB	= r3			;temporary register tempB
.def	tempC	= r4			;temporary register tempC

.def	temp	= r16			;temporary register temp
.def	tempL	= r17			;temporary register tempL
.def	tempM	= r18			;temporary register tempM
.def	tempH	= r19			;temporary register tempH
.def	Cnt	= r20			;temporary register Cnt
.def	CMD	= r21			;temporary register tempL

.def	Flags	= r25			;flags
 .equ	fUpd	= 0			;timer update flag
 .equ	fSign	= 1			;sign flag

;r26,r27 used as X-register
;r28,r29 used as Y-register
;r30,r31 used as Z-register
